# Created on 26.02.2016
# author: thl
"""Routines for checking input"""

import sys

import adnite2.constants as constants


def checkclouddensity(cloudsd):
    """Check cloud density """
    ok = True
    for t in cloudsd:
        if t not in constants.CLOUDDENSITIES:
            ok = False
    return ok


def checkcloudiness(cloudsn):
    """Check cloudiness"""
    ok = True
    for t in cloudsn:
        if t not in constants.CLOUDAMOUNTS:
            ok = False
    return ok


def checkground(ground):
    """Check ground data"""
    ok = True
    for t in ground:
        tc = t.upper()
        if tc not in constants.GROUNDDATA:
            ok = False
    return ok


def checkcloudandgrounddata(parameters):
    """Check cloud-input-data and convert cloud type and density to cloud
    albedo."""
    cloudsd = parameters['cloudsd']
    cloudsdx = parameters['cloudsdx']
    cloudsn = [float(d) for d in parameters['cloudsn']]
    cloudsnx = [float(d) for d in parameters['cloudsnx']]
    ground = parameters['ground']
    groundx = parameters['groundx']
    lentimeline = len(parameters['times'])
    if not checkclouddensity(cloudsd):
        print("ERROR: wrong cloud density data", file=sys.stderr)
        print("Check inputfile", file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('cloudsd: ', cloudsd, file=sys.stderr)
        sys.exit(-1)
    if not checkclouddensity(cloudsdx):
        print("ERROR: wrong cloud density data", file=sys.stderr)
        print("Check inputfile", file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('cloudsdx: ', cloudsdx, file=sys.stderr)
        sys.exit(-1)
    if not checkcloudiness(cloudsn):
        print("ERROR: wrong cloudiness data", file=sys.stderr)
        print("Check inputfile", file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('cloudsn: ', cloudsn, file=sys.stderr)
        sys.exit(-1)
    if not checkcloudiness(cloudsnx):
        print("ERROR: wrong cloudiness data", file=sys.stderr)
        print("Check inputfile", file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('cloudsn: ', cloudsn, file=sys.stderr)
        sys.exit(-1)
    if not checkground(ground):
        print("ERROR: wrong ground data", file=sys.stderr)
        print("Check inputfile", file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('ground: ', ground, file=sys.stderr)
        sys.exit(-1)
    if not checkground(groundx):
        print("ERROR: wrong ground data", file=sys.stderr)
        print("Check inputfile", file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('groundx: ', groundx, file=sys.stderr)
        sys.exit(-1)
    if len(cloudsd) != lentimeline:
        print('ERROR: not maching number of cloud data', file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('times: ', parameters['times'], file=sys.stderr)
        print('cloudsd', cloudsd, file=sys.stderr)
        sys.exit(-1)
    if len(cloudsdx) != lentimeline:
        print('ERROR: not maching number of cloud data', file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('times: ', parameters['times'], file=sys.stderr)
        print('cloudsdx', cloudsdx, file=sys.stderr)
        sys.exit(-1)
    if len(cloudsn) != lentimeline:
        print('ERROR: not maching number of cloud data', file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('times: ', parameters['times'], file=sys.stderr)
        print('cloudsn', cloudsn, file=sys.stderr)
        sys.exit(-1)
    if len(cloudsnx) != lentimeline:
        print('ERROR: not maching number of cloud data', file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('times: ', parameters['times'], file=sys.stderr)
        print('cloudsnx', cloudsnx, file=sys.stderr)
        sys.exit(-1)
    if len(ground) != lentimeline:
        print('ERROR: not maching number of ground data', file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('times: ', parameters['times'], file=sys.stderr)
        print('ground: ', ground, file=sys.stderr)
        sys.exit(-1)
    if len(ground) != lentimeline:
        print('ERROR: not maching number of ground data', file=sys.stderr)
        print("Also check for trailing semicolon ';' without further data",
              file=sys.stderr)
        print('times: ', parameters['times'], file=sys.stderr)
        print('groundx: ', groundx, file=sys.stderr)
        sys.exit(-1)
    return True


if __name__ == '__main__':
    pass
