# Created on 2016-03-15
"""Called by adnite2-run.

Try *adnite2-run -h*for help.
"""

import argparse
import datetime
import sys

import adnite2.constants as constants
import adnite2.graphics as graphics
import adnite2.inout as inout
import adnite2.rsttable as rsttable
import adnite2.version as version


DESCRIPTION = u"Calculate Night Vision Data"
INPUTFILEHELP = u"""inputfile with all relevant input data.The file must be
a csv-file with special keywords. You should have got a copy of a sample file
to see the format."""
BASICSHELP = u"""calculate basic astronomical information for a location and
a date or a date window. Input is read from an inputfile."""
MOONHELP = u"""calculates moon position, phase and relative brightness
for a location, a time window and an given environment.
Input is read from an inputfile."""
ILLUMHELP = u"""calculates illuminance for a location, a time window
and an given environment. Input is read from an inputfile."""
GRAPHHELP = u"""calculates basics astronomical data, moon-data and illuminance
data for a given location, time window and environment. Input is read from an
inputfile."""
RSTHELP = u"""Calculates base, moon and illumination data and outputs it
to an appropriate ReStructuredText-file"""
VERSIONHELP = u'show program version number and copyright notice and exit'
VERSION = version.version('Program', printit=False)


def main():
    """Main entry point for the script."""
    parser = argparse.ArgumentParser(description=DESCRIPTION)
    parser.add_argument('--version', help=VERSIONHELP,
                        action='version', version=VERSION)
    parser.add_argument('inputfile', help=INPUTFILEHELP)
#     parser.add_argument('-b', '--basics', action='store_true',
#                         help=BASICSHELP)
#     parser.add_argument('-m', '--moon', action='store_true', help=MOONHELP)
#     parser.add_argument('-i', '--illuminance', action='store_true',
#                         help=ILLUMHELP)
    parser.add_argument('-g', '--graphics', action='store_true',
                        help=GRAPHHELP)
    parser.add_argument('-r', '--rst', action='store_true',
                        help=RSTHELP)
    args = parser.parse_args()

    print(constants.SEPARATOR0)
    version.version('Program')
    print('Timestamp (UTC): ',
          datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"))
    print("Read input from file: ", args.inputfile)
    parms = constants.read_parameters(args.inputfile)
    print(constants.SEPARATOR1)
    inout.print_parameters(parms, 'Input-file "' + args.inputfile + '"')
#     if args.basics:
#         pass
#     elif args.moon:
#         pass
#     elif args.illuminance:
#         pass
    if args.graphics:
        sys.exit(graphics.__main__(parms))
    elif args.rst:
        sys.exit(rsttable.__main__(parms))
    else:
        sys.exit(version.version('Program', printit=False))
    return sys.exit()


if __name__ == '__main__':
    sys.exit(main())
