# Created on 2016-03-15
# dietmar.thaler@posteo.at

import sqlite3
import sys

from adnite2.version import version


def insert_replace_station(database, station, name,
                           lat_dd, lat_mm, lat_ss,
                           lon_dd, lon_mm, lon_ss,
                           elevation):
    insertreplace = "INSERT OR REPLACE "
    insertss = "INTO stationlist VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)"
    insertcommand = insertreplace + insertss
    conn = sqlite3.connect(database)  # @UndefinedVariable
    indicator = station
    # name
    latd = int(lat_dd)
    latm = int(lat_mm)
    lats = int(lat_ss)
    lond = int(lon_dd)
    lonm = int(lon_mm)
    lons = int(lon_ss)
    elev = int(elevation)
    c = conn.cursor()
    insertl = (indicator,
               name,
               latd, latm, lats,
               lond, lonm, lons,
               elev)
    c.execute(insertcommand, insertl)
    conn.commit()
    conn.close()
    return True


def delete_station(database, station):
    delcmd = "DELETE FROM stationlist WHERE indicator=?"
    conn = sqlite3.connect(database)  # @UndefinedVariable
    c = conn.cursor()
    insertl = (station,)
    c.execute(delcmd, insertl)
    conn.commit()
    conn.close()


def read_single_station(
        locationindicator, stationdatabase='./stations.sqlite'):
    """Reads from an appropriate sqlite3-Database *db* the location
         indicator *locindicator* and its geographical data and
         returns a dictionary with the form {'station': string,
         'name': string, 'lat': float, 'lon': float, 'elev': float}
    """
    db = stationdatabase
    locind = locationindicator
    try:
        con = sqlite3.connect(db)
        c = con.cursor()
    except sqlite3.Error as e:
        print("ERROR in, 'read_single_station': cannot connect with ",
              db, file=sys.stderr)
        sys.exit(e)
    try:
        c.execute('SELECT * FROM stationlist WHERE indicator=?',
                  (locind,))
    except sqlite3.Error as e:
        print("ERROR in 'stationlistdb': " + db +
              " is no datatbase or not the correct station database",
              file=sys.stderr)
        sys.exit(e)
    l = c.fetchone()
    con.close()
    if l is None:
        print("ERROR: Station " + locind + " is not in datatbase " + db)
        d = None
    else:
        d = {'station': l[0], 'name': l[1],
             'lat': (l[2] + l[3] / 60.0 + l[4] / 3600.0),
             'lon': (l[5] + l[6] / 60.0 + l[7] / 3600.0),
             'elev': l[8]}
    return d


def read_station_list_by_wildcard(database, searchstring):
    """Returns a list of station data that contain searchstring (regular
    expression)"""
    srchcmd = r"SELECT * FROM stationlist WHERE (upper(indicator) GLOB upper( ? ) OR upper(name) GLOB upper( ? ))"
    con = sqlite3.connect(database)  # @UndefinedVariable
    c = con.cursor()
    c.execute(srchcmd.upper(), (searchstring.upper(), searchstring.upper()))
    l = c.fetchall()
    con.close()
    return l


def station_in_db(stationdatabase, station):
    """Checks if station in database

    stationdatabase -- sqlite station database
    station -- station name

    Returns logical 'True' or 'False'
    """
    return not (read_single_station(station, stationdatabase) is None)


def get_all_stations(stationdatabase):
    """Returns a (ascii-)list of all station indicators  in database"""
    db = stationdatabase
    try:
        con = sqlite3.connect(db)
        c = con.cursor()
    except sqlite3.Error as e:
        print("ERROR in, 'db_read_stationlist': cannot connect with ",
              db, file=sys.stderr)
        sys.exit(e)
    try:
        c.execute('SELECT indicator FROM stationlist')
    except sqlite3.Error as e:
        print("ERROR in 'stationlistdb': " + db +
              " is no datatbase or not the correct station database",
              file=sys.stderr)
        sys.exit(e)
    ll = c.fetchall()
    con.close()
    statlist = [str(l[0]) for l in ll]
    return statlist


if __name__ == '__main__':
    version('Module')
