# Created 2016-03-01 by D.Th.
# Modified 2016-04-15 by D.Th.
# Modified 2016-05-09 by D.Th.
"""adnite2 - Graphical User Interface - Improved"""

from PIL import ImageTk, Image
import datetime
import glob
import os
import re
import shutil
import sys

import adnite2.ToolTip as ToolTip
import adnite2.constants as constants
import adnite2.db as db
import adnite2.inout as inout
import adnite2.version as version
import os.path as op
import subprocess as sp
import tkinter as tk
import tkinter.filedialog as tkd
import tkinter.font as tkfont
import tkinter.messagebox as tkmb
import tkinter.scrolledtext as st
import tkinter.ttk as ttk


FONT = "Sans"  # "UbuntuNormal"
FONT2 = "Mono"  # "UbuntuMono"
FONTSIZE0 = "13"  # titles
FONTSIZE1 = "9"  # texts, ...
FONTSIZE2 = "8"  # standard text size if not overwritten
BOLD = "bold"
NORMAL = "normal"
COLWIDTH0 = 32
COLWIDTH1 = 10
COLWIDTH2 = 12
BUTTONWIDTH = 7
LOGOWIDTH = 71
LOGOHEIGHT = 71
GUIGROUNDLIST = list(constants.GROUNDDATA)  # [0:]
# times
today = datetime.datetime.utcnow()
heute = today.strftime("%Y-%m-%d")
jetzt = today.strftime("%Y%m%d_%H%M%S")

# fixed path and filenames
CONFDIR = constants.USER_CONFIG_DIRECTORY
configfilen = constants.USER_CONFIGFILENAME
PATH2PRG = os.path.dirname(sys.argv[0])
database = op.join(CONFDIR, 'stations.sqlite')
logdir = op.join(CONFDIR, 'logfiles')
logfile = op.join(logdir, 'adnite2.log.' + jetzt)
logopath = op.expanduser(constants.LOGOPATH)
logobasname = op.basename(logopath)
rsttemplate = op.join(CONFDIR, 'template.rst')
stylesheet = op.join(CONFDIR, 'default.css')
defaultparamfile = op.join(CONFDIR, 'default.prm')

#  o p e n   l o g f i l e
logf = open(logfile, mode='w')  # overwrite exiting!
print(79 * "=", file=logf)
print("adnite2 - Logfile", file=logf)
print("Opened at:  ", datetime.datetime.utcnow(), ' UTC', file=logf)
print(79 * "=", file=logf)
logf.flush()

#  read config-file
configparms = constants.read_parameters(
    configfilen,
    allowedstrings=constants.CONFIG_PARAM_STRINGS,
    allowedlists=constants.CONFIG_PARAM_LISTS)
inout.print_parameters(configparms, header='Config-Parameters',
                       outfile=logf)

# redefine font sizes from config-paramter file if available:
try:
    FONTSIZE0 = configparms['fontsize0']
    FONTSIZE1 = configparms['fontsize1']
    FONTSIZE2 = configparms['fontsize2']
except KeyError:
    print('Warning - adnite2-gui: Font Size set to defaults')
    print('Set fontsize0, fontsize1, fontsize2 in ~/.adnite2/default.cfg')
    print(FONTSIZE0, FONTSIZE1, FONTSIZE2)


# directories and paths from config-file
station = configparms['favorites'][0]
workdir = op.abspath(op.expanduser(configparms['workdir']))
outputdir = op.join(workdir, station)
paramfile = op.join(outputdir, 'params.prm')
helpsite = configparms['helpsite']
statfavorites = configparms['favorites']
outputdir = op.join(workdir, station)

# half fixed paths
basicsname = 'basics.csv'
moonname = 'moon.csv'
illumname = 'illum.csv'
moongname = 'moon.png'
illumgname = 'illum.png'
htmlname = 'adnite.html'
pdfname = 'adnite.pdf'
rstname = 'adnite.rst'
rsttable = 'adnite_table.rst'
RST2HTML = configparms['rst2html']
RST2PDF = configparms['rst2pdf']
COREPRG = configparms['adnite']
VIEWER = configparms['viewer']
BROWSER = configparms['browser']
PDFVIEWER = configparms['pdfviewer']

#  i n i t i a l i z e   v a r i a b l e s   a n d   a r r a y s
# defaults to the first station in the favorites
stimes = ["{0:02d}:00".format(ii % 24) for ii in range(15, 32, 2)]
NTIMES = len(stimes)
cloudsn = [constants.CLOUDAMOUNTS[0] for n in range(NTIMES)]
cloudsnx = [constants.CLOUDAMOUNTS[-1] for n in range(NTIMES)]
cloudsd = [constants.CLOUDDENSITIES[0] for n in range(NTIMES)]
cloudsdx = [constants.CLOUDDENSITIES[-1] for n in range(NTIMES)]
ground = [constants.GROUNDDATA[0] for n in range(NTIMES)]
groundx = [constants.GROUNDDATA[-1] for n in range(NTIMES)]

# ToolTip-Texts
tt_TIMES = """Enter a valid UTC-time: HH:MM, e.g. 17:30.

The whole list of times must be monotonic increasing and must not span 24 hours or more.

N.B.: Time 24:00 not allowed, take 00:00 instead!"""

tt_STATION = ("""Enter a valid location indicator.
The full set is defined by entries in the station data base.
Click 'Edit DB' to search in the station database or enter your own private station.""")


class adnite:

    """Master Class for the GUI"""

    def __init__(self, root):
        """Defines the form"""
        # Set the default font size to FONTSIZE2
        default_font = tkfont.nametofont("TkDefaultFont")
        default_font.configure(size=FONTSIZE2)
        global outputdir, workdir, station, paramfile
        self.root = root
        root.option_add("*Font", default_font)
        root.protocol('WM_DELETE_WINDOW', self.exit_gui)
        root.resizable(width=False, height=False)
        # Initialize tkinter single string variables.
        self.v_workdir = tk.StringVar()
        self.v_workdir.set(workdir)
        self.v_paramfile = tk.StringVar()
        self.v_paramfile.set(paramfile)
        self.v_outputdir = tk.StringVar()
        self.v_outputdir.set(outputdir)
        self.v_heute = tk.StringVar()
        self.v_heute.set(heute)
        self.v_station = tk.StringVar()
        self.v_station.set(station)
        # status windows scrolled text object needed soon
        self.scr_txt1 = st.ScrolledText(
            master=self.root,
            wrap=tk.WORD,
            width=124,
            height=7,
            state=tk.NORMAL,
            font=(FONT2, FONTSIZE1, NORMAL),
            background='lightgrey')
        # Initialize tkinter string arrays
        self.v_times_array = self.make_tkstring_array(stimes)
        self.v_cloudsn_array = self.make_tkstring_array(cloudsn)
        self.v_cloudsnx_array = self.make_tkstring_array(cloudsnx)
        self.v_cloudsd_array = self.make_tkstring_array(cloudsd)
        self.v_cloudsdx_array = self.make_tkstring_array(cloudsdx)
        self.v_ground_array = self.make_tkstring_array(ground)
        self.v_groundx_array = self.make_tkstring_array(groundx)
        # Register validation functions
        valdnclouds = root.register(self.onNcloudsValidate)
        valddensity = root.register(self.onDensityValidate)
        valdground = root.register(self.onGroundValidate)
        valdtime = root.register(self.onTimeValidate)

        # D E F I N E   W I D G E T S   #######################
        # Logo
        image = Image.open(logopath)
        image = image.resize((LOGOWIDTH, LOGOHEIGHT), Image.ANTIALIAS)
        img = ImageTk.PhotoImage(image)
        root.tk.call('wm', 'iconphoto', root._w, img)
        l_logo = tk.Label(image=img)
        # Title
        l_title = tk.Label(text=constants.TITLEGUI,
                           font=(FONT, FONTSIZE0),
                           width=COLWIDTH0)
        # Help
        self.b_help = tk.Button(self.root,
                                text=" Help ",
                                width=BUTTONWIDTH,
                                command=self.help)
        ToolTip.ToolTip(self.b_help,
                        text='Help')
        # About
        self.b_about = tk.Button(self.root,
                                 text=" About ",
                                 width=BUTTONWIDTH,
                                 command=self.about)
        ToolTip.ToolTip(self.b_about,
                        text='About AdNite2')
        # Select workdir
        self.b_workdir = tk.Button(
            root, text="Work-Dir", width=BUTTONWIDTH + 1,
            command=self.get_workdir)
        ToolTip.ToolTip(self.b_workdir,
                        text='Select the work directory (parent dir. of all output directories')
        self.l_workdir = tk.Label(text=self.v_workdir.get(),
                                  font=(FONT, FONTSIZE1, NORMAL),
                                  justify=tk.LEFT)
        # Save config
        self.b_config = tk.Button(root, text="Save cfg",
                                  width=BUTTONWIDTH,
                                  command=self.save_config)
        ToolTip.ToolTip(self.b_config,
                        text='Save configuration file')
        # Load Parameter File
        self.b_loadparams = tk.Button(
            root, text="Load Parms", width=BUTTONWIDTH,
            command=self.load_parameter_file)
        ToolTip.ToolTip(self.b_loadparams,
                        text='Load parameters from file')
        # Save Parmameter file
        self.b_safeparams = tk.Button(root, text="Save Parms",
                                      width=BUTTONWIDTH, command=self.save_free_parameter_file)
        ToolTip.ToolTip(self.b_safeparams,
                        text='Save parameters to file')
        # Show calculated output-Directory
        self.l0_outputdir = tk.Label(text='Outp. Dir.',
                                     font=(FONT, FONTSIZE1, BOLD),
                                     width=COLWIDTH1,
                                     justify=tk.LEFT)
        self.l_outputdir = tk.Label(text=self.v_outputdir.get(),
                                    font=(FONT, FONTSIZE1, NORMAL),
                                    justify=tk.LEFT)
        # Date
        l_date = tk.Label(text='Date',
                          font=(FONT, FONTSIZE1, BOLD),
                          width=COLWIDTH1)
        self.edate = tk.Entry(
            textvariable=self.v_heute, font=(FONT, FONTSIZE1, BOLD),
            justify=tk.CENTER, width=COLWIDTH1,
            validate="focusout",
            validatecommand=self.onDateValidate)
        ToolTip.ToolTip(self.edate,
                        text='Enter valid date: YYYY-MM-DD,\n' +
                        'e.g. 2015-12-24')

        # Station
        l_station = tk.Label(text=' Station',
                             font=(FONT, FONTSIZE1, BOLD),
                             width=COLWIDTH1)
        self.estation = tk.Spinbox(root, values=statfavorites,
                                   textvariable=self.v_station,
                                   wrap=True,
                                   font=(FONT, FONTSIZE1, BOLD),
                                   justify=tk.CENTER, width=COLWIDTH1 - 2,
                                   validate="focusout",
                                   validatecommand=self.onStationValidate)
        ToolTip.ToolTip(self.estation,
                        text=tt_STATION)

        # Check station
        self.button_checkstat = tk.Button(
            root, text="Check Stat.", width=BUTTONWIDTH,
            command=self.check_station)
        ToolTip.ToolTip(self.button_checkstat,
                        text='Check station entry in\n' +
                        database + '\nand show coordinates etc..')

        # Edit (Add to and delete from) station database
        self.button_db = tk.Button(
            root, text='Edit DB', width=BUTTONWIDTH,
            command=self.edit_db, bg='lavender')
        ToolTip.ToolTip(self.button_db,
                        text="List stations or add/delete 'private' stations to/from\n" +
                        "the station data base\n" + database)

        # Times
        l_times = tk.Label(
            text='UTC',
            font=(FONT, FONTSIZE1, BOLD),
            width=COLWIDTH1)
        self.etimes_array = []
        kk = 0
        for vt in self.v_times_array:
            self.etimes_array.append(tk.Entry(textvariable=vt,
                                              font=(FONT, FONTSIZE1, NORMAL),
                                              justify=tk.CENTER, width=COLWIDTH1,
                                              validate="focusout",
                                              validatecommand=(
                                                  valdtime,
                                                  '%P',
                                                  '%W')
                                              ))
            ToolTip.ToolTip(
                self.etimes_array[kk],
                text=tt_TIMES)
            kk = kk + 1

        # N-Clouds main weather
        l_clouds = tk.Label(
            text='Cloud N/{:2.0f}'.format(constants.CLOUDAMOUNT_MAX),
            font=(FONT, FONTSIZE1, BOLD), justify=tk.LEFT,
            width=COLWIDTH1)
        self.ecloudn_array = []

        for kk in range(NTIMES):
            self.oktas = self.v_cloudsn_array[kk]
            self.ecloudn = tk.Entry(
                textvariable=self.oktas,
                font=(FONT, FONTSIZE1, NORMAL),
                justify=tk.CENTER, width=COLWIDTH1,
                validate='focusout',
                validatecommand=(valdnclouds, '%P', '%W'))
            self.ecloudn_array.append(self.ecloudn)
            ToolTip.ToolTip(
                self.ecloudn_array[kk],
                text=constants.TOOLTIP_CLOUDN)

        # Density-Clouds main weather
        l_clouddens = tk.Label(
            text='Cloud Dens',
            font=(FONT, FONTSIZE1, BOLD), justify=tk.LEFT,
            width=COLWIDTH1)
        self.ecloudd_array = []
        for kk in range(NTIMES):
            self.ecloudd_array.append(
                tk.Spinbox(values=constants.CLOUDDENSITIES,
                           textvariable=self.v_cloudsd_array[kk],
                           wrap=True,
                           font=(FONT, FONTSIZE1, NORMAL),
                           justify=tk.CENTER, width=COLWIDTH1 - 1,
                           validate='focusout',
                           validatecommand=(valddensity, '%P', '%W')))
            ToolTip.ToolTip(
                self.ecloudd_array[kk],
                text=constants.TOOLTIP_CLOUDDENS)

        # Ground main weather
        l_ground = tk.Label(
            text='Ground',
            font=(FONT, FONTSIZE1, BOLD), justify=tk.LEFT,
            width=COLWIDTH1)
        self.eground_array = []
        for kk in range(NTIMES):
            self.eground_array.append(
                tk.Spinbox(values=GUIGROUNDLIST,
                           textvariable=self.v_ground_array[kk],
                           font=(FONT, FONTSIZE1, NORMAL),
                           wrap=True,
                           justify=tk.CENTER, width=COLWIDTH1 - 1,
                           validate='focusout',
                           validatecommand=(valdground, '%P', '%W')))
            ToolTip.ToolTip(
                self.eground_array[kk],
                text=constants.TOOLTIP_GROUNDDATA)

        # N-Clouds alternate weather
        l_cloudsx = tk.Label(
            text='Cloud N/{:2.0f}'.format(constants.CLOUDAMOUNT_MAX),
            font=(FONT, FONTSIZE1, BOLD), justify=tk.RIGHT,
            width=COLWIDTH1)
        self.ecloudnx_array = []
        for kk in range(NTIMES):
            self.ecloudnx_array.append(
                tk.Entry(textvariable=self.v_cloudsnx_array[kk],
                         font=(FONT, FONTSIZE1, NORMAL),
                         justify=tk.CENTER, width=COLWIDTH1,
                         validate='focusout',
                         validatecommand=(valdnclouds, '%P', '%W')))
            ToolTip.ToolTip(
                self.ecloudnx_array[kk],
                text=constants.TOOLTIP_CLOUDN)

        # Density-Clouds alternate weather
        l_clouddensx = tk.Label(
            text='Cloud Dens',
            font=(FONT, FONTSIZE1, BOLD), justify=tk.RIGHT,
            width=COLWIDTH1)
        self.eclouddx_array = []
        for kk in range(NTIMES):
            self.eclouddx_array.append(
                tk.Spinbox(values=constants.CLOUDDENSITIES,
                           textvariable=self.v_cloudsdx_array[kk],
                           font=(FONT, FONTSIZE1, NORMAL),
                           wrap=True,
                           justify=tk.CENTER, width=COLWIDTH1 - 1,
                           validate='focusout',
                           validatecommand=(valddensity, '%P', '%W')))
            ToolTip.ToolTip(
                self.eclouddx_array[kk],
                text=constants.TOOLTIP_CLOUDDENS)

        #  Ground alternate weather
        l_groundx = tk.Label(
            text='Ground',
            font=(FONT, FONTSIZE1, BOLD), justify=tk.RIGHT,
            width=COLWIDTH1)
        self.egroundx_array = []
        for kk in range(NTIMES):
            self.egroundx_array.append(
                tk.Spinbox(values=GUIGROUNDLIST,
                           textvariable=self.v_groundx_array[kk],
                           font=(FONT, FONTSIZE1, NORMAL),
                           wrap=True,
                           justify=tk.CENTER, width=COLWIDTH1 - 1,
                           validate='focusout',
                           validatecommand=(valdground, '%P', '%W')))
            ToolTip.ToolTip(
                self.egroundx_array[kk],
                text=constants.TOOLTIP_GROUNDDATA)

        # P L A C E   W I D G E T S    ########################
        ROW = 0
        # Logo
        l_logo.grid(row=ROW, column=0)
        # Title: over full width of sheet
        l_title.grid(row=ROW, column=1,
                     columnspan=99)

        ROW = ROW + 1
        # Separator over full width of sheet
        ttk.Separator(orient='horizontal').grid(
            row=ROW, column=0, columnspan=99, sticky='ew')

        ROW = ROW + 1
        # Help
        self.b_help.grid(row=ROW, column=8, sticky='e')
        # About
        self.b_about.grid(row=ROW, column=9, sticky='e')
        # Workdir
        self.b_workdir.grid(row=ROW, column=0, sticky='w')
        self.l_workdir.grid(row=ROW, column=1,
                            columnspan=NTIMES - 2, sticky='w')

        ROW = ROW + 1
        # Save config
        self.b_config.grid(row=ROW, column=NTIMES,
                           sticky='e')
        # Load Parameter File
        self.b_loadparams.grid(row=ROW, column=NTIMES - 1, sticky='e')
        # Save Parmameter file
        self.b_safeparams.grid(row=ROW, column=NTIMES - 2, sticky='e')
        # Show calculated output-Directory
        self.l0_outputdir.grid(row=ROW, column=0,
                               columnspan=NTIMES - 3, sticky='w')
        self.l_outputdir.grid(row=ROW, column=1,
                              columnspan=NTIMES - 2, sticky='w')

        ROW = ROW + 1
        # Separator over full width of sheet
        ttk.Separator(orient='horizontal').grid(
            row=ROW, column=0, columnspan=99, sticky='ew')

        ROW = ROW + 1
        # Date
        l_date.grid(row=ROW, column=0)
        self.edate.grid(row=ROW, column=1)

        # Station
        l_station.grid(row=ROW, column=2)
        self.estation.grid(row=ROW, column=3)

        # Check station
        self.button_checkstat.grid(row=ROW, column=4)

        # Edit (Add to and delete from) station database
        self.button_db.grid(row=ROW, column=NTIMES, sticky='e')

        # Dummy label to widen field on the right just a bit
        tk.Label(text=' ',
                 font=(FONT, FONTSIZE1, BOLD),
                 width=1).grid(row=ROW, column=NTIMES + 1)

        ROW = ROW + 1
        # Separator over full width of sheet
        ttk.Separator(orient='horizontal').grid(
            row=ROW, column=0, columnspan=99, sticky='ew')

        # DISPLAY all INPUT-PARAMETERS 1st time
        ROW = ROW + 1
        # Times
        l_times.grid(row=ROW, column=0)
        kk = 0
        for vt in self.v_times_array:
            self.etimes_array[kk].grid(row=ROW, column=kk + 1)
            kk = kk + 1

        ROW = ROW + 1
        # Separator over full width of sheet
        ttk.Separator(orient='horizontal').grid(
            row=ROW, column=0, columnspan=99, sticky='ew')

        ROW = ROW + 1
        # Weather main
        tk.Label(
            text='MAIN WX',
            font=(FONT, FONTSIZE1, BOLD), justify=tk.LEFT,
            width=15).grid(row=ROW, column=0, columnspan=2, sticky='ew')

        ROW = ROW + 1
        # N-Clouds main weather
        l_clouds.grid(row=ROW, column=0)
        for kk in range(NTIMES):
            self.ecloudn_array[kk].grid(row=ROW, column=kk + 1)

        ROW = ROW + 1
        # Density-Clouds main weather
        l_clouddens.grid(row=ROW, column=0)
        for kk in range(NTIMES):
            self.ecloudd_array[kk].grid(row=ROW, column=kk + 1)

        ROW = ROW + 1
        # Ground main weather
        l_ground.grid(row=ROW, column=0)
        for kk in range(NTIMES):
            self.eground_array[kk].grid(row=ROW, column=kk + 1)

        ROW = ROW + 1
        # Separator over full width of sheet
        ttk.Separator(orient='horizontal').grid(
            row=ROW, column=0, columnspan=99, sticky='ew')

        ROW = ROW + 1
        # Alternate weather
        tk.Label(
            text='ALTERNATE WX',
            font=(FONT, FONTSIZE1, BOLD), justify=tk.LEFT,
            width=15).grid(row=ROW, column=0, columnspan=2, sticky='ew')

        ROW = ROW + 1
        # N-Clouds alternate weather
        l_cloudsx.grid(row=ROW, column=0, sticky='w')
        for kk in range(NTIMES):
            self.ecloudnx_array[kk].grid(row=ROW, column=kk + 1)

        ROW = ROW + 1
        # Density-Clouds alternate weather
        l_clouddensx.grid(row=ROW, column=0, sticky='w')
        for kk in range(NTIMES):
            self.eclouddx_array[kk].grid(row=ROW, column=kk + 1)

        ROW = ROW + 1
        #  Ground alternate weather
        l_groundx.grid(row=ROW, column=0, sticky='w')
        for kk in range(NTIMES):
            self.egroundx_array[kk].grid(row=ROW, column=kk + 1)

        ROW = ROW + 1
        # Separator over full width of sheet
        ttk.Separator(orient='horizontal').grid(row=ROW, column=0,
                                                columnspan=99, sticky='ew')

        ROW = ROW + 1
        #  Execute
        tk.Label(
            text='EXECUTE',
            font=(FONT, FONTSIZE1, BOLD), justify=tk.LEFT,
            width=COLWIDTH1).grid(row=ROW, column=0, columnspan=2)

        ROW = ROW + 1

        # delete log files
        self.button2 = tk.Button(root, text="Del log",
                                 width=BUTTONWIDTH, command=self.del_logs)
        self.button2.grid(row=ROW, column=7)
        ToolTip.ToolTip(self.button2,
                        text='Delete old logfiles')

        # execute all command
        self.button3 = tk.Button(root, text="Execute",
                                 width=BUTTONWIDTH,
                                 command=self.execute_all,
                                 bg='palegreen')
        self.button3.grid(row=ROW, column=0)
        ToolTip.ToolTip(
            self.button3,
            text='Do all calculations and safe results to the outputdirectory')

        # watch html
        self.button4 = tk.Button(root, text="View HTML",
                                 width=BUTTONWIDTH, command=self.view_html)
        self.button4.grid(row=ROW, column=1)
        ToolTip.ToolTip(self.button4,
                        text='View the html result in the broser')

        # watch pdf
        self.button5 = tk.Button(root, text="View PDF",
                                 width=BUTTONWIDTH, command=self.view_pdf)
        self.button5.grid(row=ROW, column=2)
        ToolTip.ToolTip(self.button5,
                        text='View the pdf result in the pdf-viewer')

        # view logfile
        self.buttonL = tk.Button(root, text="Logfile", width=BUTTONWIDTH,
                                 command=self.view_logfile)
        self.buttonL.grid(row=ROW, column=8)
        ToolTip.ToolTip(self.buttonL,
                        text='In case of troubles ' +
                        'view the log-file for further infos')

        # exit command
        self.buttonx = tk.Button(root, text="Exit", width=BUTTONWIDTH,
                                 command=self.exit_gui,
                                 bg='pink')
        self.buttonx.grid(row=ROW, column=9)
        ToolTip.ToolTip(self.buttonx,
                        text='Guess what?')

        ROW = ROW + 1
        # Separator over full width of sheet
        ttk.Separator(orient='horizontal').grid(
            row=ROW, column=0, columnspan=99, sticky='ew')

        # INFO-Header
        ROW = ROW + 1
        tk.Label(
            text='INFOS ',  # Status',
            font=(FONT, FONTSIZE1, BOLD), justify=tk.LEFT,
            width=COLWIDTH1).grid(row=ROW, column=0, columnspan=2)
        tk.Label(
            text='(for more information see log-file)',  # Status',
            font=(FONT, FONTSIZE1, NORMAL), justify=tk.LEFT).grid(
            row=ROW, column=2, columnspan=3, sticky='w')

        # Info area
        ROW = ROW + 1
        self.scr_txt1.grid(row=ROW, column=0,
                           columnspan=NTIMES + 1)
        self.scr_txt1.configure(state=tk.DISABLED)

        # Empty last row for beauty
        ROW = ROW + 99
        tk.Label(text='  ',
                 font=(FONT, FONTSIZE1, BOLD), justify=tk.LEFT,
                 width=COLWIDTH1).grid(row=ROW, column=0,
                                       columnspan=NTIMES + 1)

        try:
            # Load defaults if available
            self.load_default_parameter_file()
        except:
            pass
        self.check_create_workdir()
        root.mainloop()

        return None

    def log_message(self, message):
        """Write messages to the logfile logf and the info-area"""
        txt = self.scr_txt1
        nun = datetime.datetime.utcnow().strftime("%H:%M:%S: ")
        fullm = 79 * '-' + "\n"
        fullm = fullm + 'adnite2-gui ' + nun + message + '\n'
        # status area
        txt.configure(state=tk.NORMAL)
        txt.insert(tk.END, fullm)
        txt.see(tk.END)
        txt.configure(state=tk.DISABLED)
        # log file
        print(fullm, end='', file=logf)
        logf.flush()
        return

    def onDateValidate(self):
        """Check the date at input"""

        def false_date():
            self.log_message('ERROR - Date (yyyy-mm-dd) not ok: ' + dts)
            self.edate.selection_clear()
            self.edate.focus_set()
            self.edate.select_range(0, tk.END)
            return False

        REDATE = r'^[0-9]{4}-[0-9]{2}-[0-9]{2}$'
        dts = self.v_heute.get()
        valid = True
        if re.search(REDATE, dts):
            try:
                datetime.datetime.strptime(dts, "%Y-%m-%d")
                self.log_message('Date ok: ' + dts)
            except ValueError:
                valid = false_date()
        else:
            valid = false_date()
        return valid

    def onStationValidate(self):
        """Check if station in database"""
        global workdir, outputdir, station, paramfile
        global v_outputdir, l_outputdir

        # sl = db.get_all_stations(database)
        station = self.v_station.get().strip().upper()
        valid = db.station_in_db(database, station)
        if valid:
            outputdir = op.join(workdir, station)
            # inout.create_dirs(outputdir)
            paramfile = op.join(outputdir, 'parameters.prm')
            self.v_outputdir.set(outputdir)
            self.l_outputdir.config(text=self.v_outputdir.get(),
                                    font=(FONT, FONTSIZE1, NORMAL),
                                    justify=tk.LEFT)
            self.log_message('Outputdirectory: ' + self.v_outputdir.get())
            pass
        else:
            errmsg = 'ERROR - Station not in database'
            self.log_message(errmsg + ': ' + station)
            tkmb.showerror(errmsg, errmsg + ': ' + station)
            self.estation.selection_clear()
            self.estation.focus_set()
            self.estation.selection('range', 0, tk.END)
        return valid

    def onNcloudsValidate(self, nclouds, widget):
        """Check if cloud nclouds are in range"""
        try:
            valid = (float(nclouds) in constants.CLOUDAMOUNTS)
        except:
            valid = False
        wx = self.root.nametowidget(widget)
        if not valid:
            errmsg0 = 'ERROR - Invalid cloudiness.\n'
            errmsg = (errmsg0 +
                      'Clouds must be in range {:2.0f}..{:2.0f}'.format(
                          min(constants.CLOUDAMOUNTS),
                          max(constants.CLOUDAMOUNTS)))
            self.log_message(errmsg)
            tkmb.showerror(errmsg0, errmsg)
            self.estation.selection_clear()
            self.estation.focus_set()
            self.estation.selection('range', 0, tk.END)
            wx.selection_clear()
            wx.focus_set()
            wx.select_range(0, tk.END)
        return valid

    def onDensityValidate(self, density, widget):
        """Check if cloud denisties are in range 0..3"""
        valid = (density in constants.CLOUDDENSITIES)
        wx = self.root.nametowidget(widget)
        if not valid:
            errmsg0 = 'ERROR - Invalid cloud density.\n'
            errmsg = (
                errmsg0 +
                'Cloud densities must be in range {:}..{:}'.format(
                    constants.CLOUDDENSITIES[0],
                    constants.CLOUDDENSITIES[-1]))
            self.log_message(errmsg)
            tkmb.showerror(errmsg0, errmsg)
            wx.selection_clear()
            wx.focus_set()
            wx.selection('range', 0, tk.END)
        return valid

    def onGroundValidate(self, ground, widget):
        """Check if cloud oktas are in range 0..8"""
        valid = (ground.upper() in constants.GROUNDDATA)
        wx = self.root.nametowidget(widget)
        if not valid:
            errmsg0 = 'ERROR - Invalid ground type.\n'
            errmsg = (
                errmsg0 +
                'Ground type must be out of list: ' + ", ".join(constants.GROUNDDATA))
            self.log_message(errmsg)
            tkmb.showerror(errmsg0, errmsg)
            wx.selection_clear()
            wx.focus_set()
            wx.selection('range', 0, tk.END)
        return valid

    def onGroundValidate__(self, ground, widget):
        """Check if cloud oktas are in range 0..8"""
        valid = (ground.upper() in constants.GROUNDDATA)
        wx = self.root.nametowidget(widget)
        if valid:
            pass
            # self.log_message('Ground data OK: ' + ground)
        else:
            self.log_message('ERROR: invalid ground data: ' + ground)
            wx.selection_clear()
            wx.focus_set()
            wx.selection('range', 0, tk.END)
        return valid

    def onTimeValidate(self, time, widget):
        """Check if cloud oktas are in range 0..8"""
        try:
            datetime.datetime.strptime(time, "%H:%M")
            valid = True
        except ValueError:
            valid = False
        wx = self.root.nametowidget(widget)
        if valid:
            pass
            # self.log_message('Time OK: ' + time)
        else:
            self.log_message('ERROR: invalid time: ' + time)
            wx.selection_clear()
            wx.focus_set()
            wx.select_range(0, tk.END)
        return valid

    def log_button_click(self, message):
        """Heritage function for log_message"""
        self.log_message(message)

    def edit_db(self):
        # Start adnitedbedit
        sp.call(['adnite2dbedit', '-a', 'adnite2'])
        return

    def exit_gui(self):
        """Exit GUI"""
        OK = tkmb.askokcancel("Exit adnite2", "Exit really?",
                              default=tkmb.CANCEL)
        if OK:
            self.log_button_click('close_window - OK')
            print(79 * "=", file=logf)
            print("adnite3 - Logfile", file=logf)
            print("Closed at: ", datetime.datetime.utcnow(), ' UTC', file=logf)
            print(79 * "=", file=logf)
            logf.close()
            # Dirty hack to remove the silly 'nohup.out'-File
            try:
                os.remove('nohup.out')
            except:
                pass
            self.root.destroy()
        else:
            self.log_button_click("close_window - CANCEL")
        return

    def get_workdir(self):
        """Sets the name of an the work directory"""
        global workdir, v_workdir, l_workdir, outputdir
        self.log_button_click('get_workdir')
        wdir = tkd.askdirectory(initialdir=workdir,
                                title='Please select the work directory')
        if len(wdir) > 0:
            workdir = wdir
            self.v_workdir.set(workdir)
            self.l_workdir.config(text=self.v_workdir.get())
        outputdir = op.join(workdir, station)
        self.onStationValidate()
        return

    def form_from_parm(self, parms):
        """Make the gui-form from inputparameters"""
        self.v_station.set(parms['station'])
        # Times
        self.etimes_array = []
        for kk, __ in enumerate(self.v_times_array):
            self.v_times_array[kk].set(parms['times'][kk])
            self.v_cloudsn_array[kk].set(parms['cloudsn'][kk])
            self.v_cloudsnx_array[kk].set(parms['cloudsnx'][kk])
            self.v_cloudsd_array[kk].set(parms['cloudsd'][kk])
            self.v_cloudsdx_array[kk].set(parms['cloudsdx'][kk])
            self.v_ground_array[kk].set(parms['ground'][kk])
            self.v_groundx_array[kk].set(parms['groundx'][kk])
        return

    def load_parameter_file(self):
        self.log_message('load_parameter_file')
        self.log_button_click('load_parameter_file')
        infile = tkd.askopenfilename(
            title="Load parameters from file",
            filetypes=[('Text files', '*.prm'), ('All files', '*.*')],
            initialdir=workdir)
        if len(infile) > 0:
            parms = constants.read_parameters(infile)
            inout.print_parameters(parms, 'New read inputparms', logf)
            self.form_from_parm(parms)
        return

    def load_default_parameter_file(self):
        self.log_message('load_parameter_file')
        self.log_button_click('load_parameter_file')
        if op.exists(defaultparamfile):
            parms = constants.read_parameters(defaultparamfile)
            inout.print_parameters(parms, 'New read inputparms', logf)
            self.form_from_parm(parms)
        return

    def make_parameter_dict(self):
        """Make a dictionary of all relevant adnite parameters."""
        self.log_message('parm_from_form')
        parameters = {}
        parameters['stationdb'] = os.path.abspath(database)
        parameters['outdir'] = os.path.abspath(outputdir)
        parameters['station'] = self.estation.get()
        parameters['date1'] = self.edate.get()
        parameters['basics_outfilen'] = op.join(outputdir, basicsname)
        parameters['moon_outfilen'] = op.join(outputdir, moonname)
        parameters['illum_outfilen'] = op.join(outputdir, illumname)
        parameters['moon_goutfilen'] = op.join(outputdir, moongname)
        parameters['illum_goutfilen'] = op.join(outputdir, illumgname)
        parameters['rsttable'] = op.join(outputdir, rsttable)
        parameters['rsttemplate'] = rsttemplate
        parameters['rstfile'] = op.join(outputdir, rstname)
        parameters['times'] = []
        for kk in range(NTIMES):
            parameters['times'].append(self.v_times_array[kk].get())
        parameters['cloudsn'] = []
        for kk in range(NTIMES):
            parameters['cloudsn'].append(self.ecloudn_array[kk].get())
        parameters['cloudsd'] = []
        for kk in range(NTIMES):
            parameters['cloudsd'].append(self.ecloudd_array[kk].get())
        parameters['ground'] = []
        for kk in range(NTIMES):
            parameters['ground'].append(self.eground_array[kk].get())
        parameters['cloudsnx'] = []
        for kk in range(NTIMES):
            parameters['cloudsnx'].append(self.ecloudnx_array[kk].get())
        parameters['cloudsdx'] = []
        for kk in range(NTIMES):
            parameters['cloudsdx'].append(self.eclouddx_array[kk].get())
        parameters['groundx'] = []
        for kk in range(NTIMES):
            parameters['groundx'].append(self.egroundx_array[kk].get())
        return parameters

    def save_free_parameter_file(self):
        """Save parameter file freely for later use.

        GUI interaction.
        """
        self.onDateValidate()
        self.onStationValidate()
        self.log_message('Save_standard_parameter_file')
        parameters = self.make_parameter_dict()
        inout.print_parameters(parameters, 'In adnite3-Gui', outfile=logf)
        ok = True
        f = tkd.asksaveasfile(
            'w',
            defaultextension='.prm',
            title='Save parameter file for later use',
            initialdir=workdir)
        # sys.exit()
        try:
            for p in list(parameters.keys()):
                data = parameters[p]
                if isinstance(data, str):
                    print("{0:}; {1:}".format(p, data), file=f)
                else:
                    print(p, end="", file=f)
                    [print("; {0:}".format(d), end="", file=f) for d in data]
                    print(file=f)
            f.close()
            print("Parameter file saved", file=logf)
        except:
            pass
        logf.flush()
        return ok

    def save_standard_parameter_file(self):
        """Save parameter to standard file name.

        No user interaction
        """
        self.onStationValidate()
        self.onDateValidate()
        self.log_message('Save_standard_parameter_file')
        parameters = self.make_parameter_dict()
        inout.print_parameters(parameters, 'In adnite3-Gui', outfile=logf)
        ok = True
        try:
            inout.create_dirs(outputdir)
            self.log_message(
                'Output directory ' +
                outputdir +
                ' (now) existing')
            f = open(paramfile, mode='w')
        except IOError as e:
            print(
                "ERROR in adnite2-gui.save_standard_parameter_file:",
                file=logf)
            print(e, file=logf)
            tkmb.showerror(
                "ERROR in adnite2-gui.save_standard_parameter_file",
                e)
            return False
        for p in list(parameters.keys()):
            data = parameters[p]
            if isinstance(data, str):
                print("{0:}; {1:}".format(p, data), file=f)
            else:
                print(p, end="", file=f)
                [print("; {0:}".format(d), end="", file=f) for d in data]
                print(file=f)
        f.close()
        print("Parameter file saved to: ", paramfile, file=logf)
        logf.flush()
        return ok

    def show_extprog_error(self, prog):
        self.log_message('show_extprog_error')
        tkmb.showerror("Exec&HTML",
                       "Error in '" + prog + "'" +
                       "\nFor details look at log file\n" + logfile)
        return

    def save_config(self):
        global configparms
        self.log_message('save_config')
        configparms['workdir'] = workdir
        try:
            f = open(configfilen, mode='w')
            inout.print_parameters(configparms, 'Config-parameters', logf)
            inout.save_parameters(configparms,
                                  constants.CONFIG_PARAM_STRINGS,
                                  constants.CONFIG_PARAM_LISTS, f)
            tkmb.showinfo(
                "Save config",
                "Config data successfully saved to: \n" + configfilen
            )
            f.close()
        except:
            tkmb.showerror(
                "Save config - Error",
                ("Config data cannot be saved to: \n" +
                 configfilen +
                 '\nNo config-directory or unsufficient privileges?')
            )
        return

    def start_all(self):
        """Does all the dull performance"""
        self.log_message('start_all')
        cmd0 = COREPRG
        # ok = True
        ok = self.save_standard_parameter_file()
        if ok:
            if sp.call([cmd0, '-g',
                        paramfile], stdout=logf,
                       stderr=logf) == 0:
                self.log_message(COREPRG + ' -g is OK')
                if sp.call([cmd0, '-r',
                            paramfile], stdout=logf,
                           stderr=logf) == 0:
                    self.log_message(COREPRG + ' -r is OK')
                else:
                    self.show_extprog_error(COREPRG + ' -r')
                    self.log_message('ERROR in ' + COREPRG + ' -r')
                    ok = False
            else:
                self.show_extprog_error(COREPRG + ' -g')
                self.log_message('ERROR in ' + COREPRG + ' -g')
                ok = False
        logf.flush()
        return ok

    def view_html(self):
        # global parameters
        global htmlfn, otputdir, htmlname
        self.log_message('watch_html')
        htmlfn = os.path.join(outputdir, htmlname)
        sp.Popen(["nohup", BROWSER, htmlfn], stderr=logf)
        return

    def view_pdf(self):
        # global parameters
        global pdffn, otputdir, pdfname
        self.log_message('watch_pdf')
        pdffn = os.path.join(outputdir, pdfname)
        sp.Popen(["nohup", PDFVIEWER, pdffn], stderr=logf)
        return

    def help(self):
        # global parameters
        global helpfile
        sp.Popen(["nohup", BROWSER, helpsite], stderr=logf)
        return

    def execute_all(self):
        self.log_message('START Execute Calculations')
        if self.start_all():
            shutil.copy(logopath, outputdir)
            parameters = self.make_parameter_dict()
            rstfn = os.path.join(parameters['outdir'], rstname)
            htmlfn = os.path.join(parameters['outdir'], htmlname)
            pdffn = op.join(outputdir, pdfname)
            f1 = open(rsttemplate, mode='r')
            s1 = f1.read()
            f1.close()
            f2 = open(rstfn, mode='w')
            print(s1, file=f2)
            f2.close()
            if sp.call([RST2HTML, '-l', 'de',
                        '--title=adnite2', '--time',
                        '--stylesheet', stylesheet, rstfn, htmlfn]) != 0:
                tkmb.showerror(
                    "rst2html - Error",
                    "Error in RST-File. Watch HTML for more Information")
            else:
                pass
            if sp.call([RST2PDF, rstfn,
                        '--header', constants.TITLEGUI,
                        '--footer', ' ###Page###/###Total###',
                        '--language', 'de',
                        '-o', pdffn]) != 0:
                tkmb.showerror(
                    "rst2pdf - Error",
                    "Error in RST-File? View PDF and Log-File for more Information")
            else:
                tkmb.showinfo("Result OK",
                              'PDF and HTML written to: \n' +
                              pdffn + '\nand\n' +
                              htmlfn)
                self.log_message('END Execute Calculations --> OK')
        return None

    def make_tkstring_array(self, sarray):
        self.log_message('make_tkstring_array')
        self.tk_stringarray = []
        for k, s in enumerate(sarray):
            self.tk_stringarray.append(tk.StringVar())
            self.tk_stringarray[k].set(s)
        return self.tk_stringarray

    def view_logfile(self):
        self.log_message('view_logfile')
        logf.flush()
        sp.Popen(["nohup", BROWSER, logfile])
        return

    def about(self):
        self.log_message('about')
        s = version.version(printit=False)
        tkmb.showinfo('About', s)
        return

    def del_logs(self):
        self.log_message('del_logs')
        flist = glob.glob(op.join(logdir, '*'))
        yc = tkmb.askokcancel('Delete Log-Files',
                              'Delete old Log-Files')
        if yc:
            for fl in flist:
                if fl != logfile:
                    os.remove(fl)
                    self.log_message('Log-File: ' + fl + ' removed')
                else:
                    self.log_message(
                        'Actual logfile ' +
                        logfile +
                        ' NOT removed')
        else:
            self.log_message('del_logs - canceled')
        return

    def check_station(self):
        """Check station and output geog. info"""
        self.onStationValidate()
        s = self.v_station.get().strip().upper()
        sd = db.read_single_station(s, database)
        if sd is None:
            self.estation.selection_clear()
            self.estation.focus_set()
            self.estation.selection('range', 0, tk.END)
            tkmb.showerror('Not in Database',
                           'Station\n' + s + '\nnot in database\n' + database)
        else:
            message = "\n{0:8s} {1:40s}  Lat:{2:8.4f}   Lon:{3:9.4f}  {4:5.0f} m".format(
                sd['station'], inout.trunc_str(sd['name'], 40), sd['lat'], sd['lon'], sd['elev'])
            self.log_message(message)
        return

    def check_create_workdir(self):
        """Does workdir exist? Create it or choose another one."""
        global workdir
        title = 'Workdir does not exist'
        msg0 = "Workdir {:} does not exist. - ".format(workdir)
        msg1 = "Create it (yes) or choose an existing one (no)?\n"
        msg2 = 'Choice is saved for later use.'
        message = msg0 + msg1 + msg2
        if not op.exists(workdir):
            if tkmb.askquestion(title, message) == tkmb.NO:
                self.get_workdir()
                self.save_config()
        inout.create_dirs(workdir)
        return


def main():
    """Main entry point for the script."""
    root = tk.Tk(className=' ' + constants.TITLEGUI)
    adnite(root)

if __name__ == "__main__":
    main()
