# Created 2016-03-01 by D.Th.
"""restucturedText-tables"""

import datetime
import sys

import adnite2.astrophysics as astrophysics
import adnite2.checker as checker
import adnite2.constants as constants
import adnite2.dattims as dattims
import adnite2.db as db
import adnite2.inout as inout
import adnite2.version as version


def __main__(parameters):
    """Main entry point for the script."""
    print(constants.SEPARATOR1)
    print("Running 'adnite2.rsttable.py'")

    try:
        stationdb = parameters['stationdb']
    except KeyError as e:
        print("ERROR in rsttable.py", file=sys.stderr)
        print("Cannot find valid database entry in inputfile",
              file=sys.stderr)
        sys.exit(e)
    try:
        station = parameters['station'].upper()
    except KeyError:
        print("WARNING: no 'station' in input file",
              file=sys.stderr)
        print("Station set to " + constants.DEFAULTSTATION,
              file=sys.stderr)
        station = constants.DEFAULTSTATION
        parameters['station'] = station
    stationdata = db.read_single_station(station, stationdb)
    try:
        pydate1 = dattims.str2pydatetime(parameters['date1'])
    except KeyError:
        print("No start date found, today's date is taken")
        now = datetime.datetime.utcnow()
        pydate1 = dattims.pydatetime2pydatetime00(now)

    try:
        pydatetimes = dattims.pydatestrtime2pydatetime_array(
            pydate1,
            parameters['times'],
            timeformat='%H:%M')
    except Exception as e:
        print('ERROR in rsttable.py: Cannot not correctly convert',
              file=sys.stderr)
        print(parameters['times'],
              file=sys.stderr)
        print('Wrong format?', files=sys.stderr)
        sys.exit(e)

    print('Station      : ', station)
    print('Date         : ', pydate1.strftime("%Y-%m-%d"))
    for d in pydatetimes:
        print(' descr. times: ', d.strftime('%b-%d  %H:%M:%S'))
    timespan = pydatetimes[-1] - pydatetimes[0]
    timesp_min = dattims.timedelta_totalseconds(timespan) // 60
    print('Timespan     : {0:} ({1:.0f} minutes)'.format(timespan, timesp_min))
    print('Timestep     : {0:} minutes'.format(constants.TIMESTEP4GRAPHS))

    if checker.checkcloudandgrounddata(parameters):
        print('Cloud and ground input data OK')
    print(constants.SEPARATOR2)
    # astro basics
    basicdata = astrophysics.calc_astro_night(stationdata, pydate1)
    print('Basic data calculated')
    relevantdata = astrophysics.extract_relevant_astro_night(basicdata)
    moondata = astrophysics.moondata(stationdata, pydatetimes)
    print('Moon data calculated')
    # illuminances
    illdata = astrophysics.illuminancedata_coarse(
        stationdata,
        pydatetimes,
        params=parameters)
    print('Illuminance data calculated')

    print(constants.SEPARATOR2)
    print('Start table output ...')
    tablefilename = inout.output_rst_table(pydatetimes,
                                           parameters,
                                           stationdata,
                                           relevantdata,
                                           moondata,
                                           illdata)
    # Trailing info
    print('RST-Table saved to: ')
    print(tablefilename)
    print(constants.SEPARATOR0)
    return


if __name__ == '__main__':
    version.version('Module')
